@@ -14,6 +14,7 @@ from askdata.security import SignUp
 from requests.adapters import HTTPAdapter
 from urllib3.util.retry import Retry
 from typing import TYPE_CHECKING
+from IPython.core.display import display, HTML
 
 if TYPE_CHECKING:
     from askdata.askdata_client import Askdata
@@ -416,16 +417,16 @@ class Agent(Insight, Channel, Catalog, Dataset):
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
         s.mount('https://', HTTPAdapter(max_retries=retries))
 
-        authentication_url = self._base_url_askdata + '/smartbot/agents/agentslug/'+ agent_slug +'/datasetslug/' + slug
-        logging.info("AUTH URL {}".format(authentication_url))
+        try:
+            authentication_url = self._base_url_askdata + '/smartbot/agents/agentslug/' + agent_slug + '/datasetslug/' + slug
+            logging.info("AUTH URL {}".format(authentication_url))
 
-        headers = {"Authorization": "Bearer" + " " + self._token}
-        response = s.get(url=authentication_url, headers=headers)
-        response.raise_for_status()
-        r = response.json()
-        if (r["datasetId"]):
+            headers = {"Authorization": "Bearer" + " " + self._token}
+            response = s.get(url=authentication_url, headers=headers)
+            response.raise_for_status()
+            r = response.json()
             return r["datasetId"]
-        else:
+        except:
             return None
 
     def _update_dataset_icon(self, dataset_id, icon_url):
@@ -823,16 +824,86 @@ class Askdata(SignUp):
         return Agent(self, slug=slug, agent_id=agent_id)
 
 
-    def get(self, query, workspace, dataset, lang="en"):
+    def get(self, query, workspace='', dataset='', lang="en"):
+
         if self._env == 'dev':
-            base_url = url_list['BASE_URL_INSIGHT_DEV']
+            base_url = url_list['BASE_URL_ASKDATA_DEV']
         elif self._env == 'qa':
-            base_url = url_list['BASE_URL_INSIGHT_QA']
+            base_url = url_list['BASE_URL_ASKDATA_QA']
         elif self._env == 'prod':
-            base_url = url_list['BASE_URL_INSIGHT_PROD']
+            base_url = url_list['BASE_URL_ASKDATA_PROD']
 
-        url = base_url + "/data/nl/result"
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
 
+        if(workspace == '' and dataset==''):
+            logging.error("Please insert a dataset or a workspace")
+        elif(workspace != '' and dataset==''):
+            dataset_id = ''
+            try:
+
+                authentication_url = base_url+ '/smartbot/agents/agentslug/' + workspace
+
+                headers = {"Authorization": "Bearer" + " " + self._token}
+                response = s.get(url=authentication_url, headers=headers)
+                response.raise_for_status()
+                r = response.json()
+                agent_id = r["agentId"]
+            except Exception as e:
+                logging.error("{}".format(e.__str__()))
+                logging.error("{} workspace not found".format(workspace))
+                return None
+
+        else:
+            # Get agent_id and dataset_id
+            try:
+                workspace=dataset.split("/")[0]
+                slug=dataset.split("/")[1]
+
+                authentication_url = base_url+'/smartbot/agents/agentslug/'+workspace + '/datasetslug/' + slug
+
+                headers = {"Authorization": "Bearer" + " " + self._token}
+                response = s.get(url=authentication_url, headers=headers)
+                response.raise_for_status()
+                r = response.json()
+                agent_id = r["agentId"]
+                dataset_id = r["datasetId"]
+            except Exception as e:
+                logging.error("{}".format(e.__str__()))
+                logging.error("{} workspace not found".format(workspace))
+                return None
+
+
+        url = base_url + "/smartbot/agents/"+agent_id+"/ask"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        body = {
+            "message": query,
+            "language": lang.lower(),
+            "userId": self.userid
+        }
+
+        response = s.post(url=url, json=body, headers=self._headers)
+        response.raise_for_status()
+        result = response.json()["result"]
+
+        #print("{} from the dataset: {} / {}".format(self.extract_fields(result[0]["attachment"]["body"][0]["details"]["columns"]), workspace, result[0]["attachment"]["body"][0]["details"]["datasetName"]))
+        display(HTML("<p>{} from the dataset: {} / {}</p>".format(self.extract_fields(result[0]["attachment"]["body"][0]["details"]["columns"]), workspace, result[0]["attachment"]["body"][0]["details"]["datasetName"])))
+        if(len(result[0]["attachment"]["body"])>1):
+            print("Filtered by: {}".format(result[0]["attachment"]["body"][1]["details"]["text"]))
+        if(len(result)>1):
+            print("{} Alternatives:".format(len(result)-1))
+            for r in result[1:]:
+                print("     {} from the dataset: {} / {}".format(self.extract_fields(r["attachment"]["body"][0]["details"]["columns"]), workspace, r["attachment"]["body"][0]["details"]["datasetName"]))
+
+        df = pd.DataFrame(data=result[0]["attachment"]["body"][0]["details"]["rows"], columns=result[0]["attachment"]["body"][0]["details"]["columns"])
+        return df
 
     def load_agents(self):
 
@@ -891,3 +962,9 @@ class Askdata(SignUp):
         r.raise_for_status()
 
         return r
+
+    def extract_fields(self, fields:list)->str:
+        column_names = ""
+        for field in fields:
+            column_names+=(field+ " , ")
+        return column_names[:-3]
